@@ -23,7 +23,7 @@ static int font_writemode = WRITEMODE_OVERWRITE;
 static int compressed_font_bg = 0;
 static int compressed_font_fg = 15;
 
-static void writecompressed(int x, int y, char *s);
+static void writecompressed(int x, int y, unsigned char *s);
 
 
 void gl_colorfont(int fw, int fh, int fg, void *_dp)
@@ -111,7 +111,7 @@ void gl_write(int x, int y, char *s)
 
 void gl_writen(int x, int y, int n, char *s)
 {
-    char *str = alloca(n + 1);
+    unsigned char *str = alloca(n + 1);
     memcpy(str, s, n);
     str[n] = 0;
     gl_write(x, y, str);
@@ -121,7 +121,7 @@ void gl_writen(int x, int y, int n, char *s)
 
 static int gl_nprintf(int sx, int sy, size_t bufs, const char *fmt, va_list args)
 {
-    char *buf;
+    unsigned char *buf;
     static int x = 0, y = 0, x_start = 0;
     int n;
 
@@ -304,7 +304,7 @@ static void expandcharacter(int bg, int fg, int c, unsigned char *bitmap)
 
 /* Write using compressed font. */
 
-static void writecompressed(int x, int y, char *s)
+static void writecompressed(int x, int y, unsigned char *s)
 {
     unsigned char *bitmap;
     int i;
