@@ -1,7 +1,8 @@
 /*
 At3d (at25) driver - Matan Ziv-Av zivav@cs.bgu.ac.il
 please report problems to me,
-if you have an alliance promotion at24, or 6422, it should
+
+If you have an alliance promotion 6422, it should
 be easy to make the driver work on it.
 
 This driver is based on the XFREE86 apm driver, developed by
@@ -10,18 +11,18 @@ Kent Hamilton and by Henrik Harmsen.
 I used the file cirrus.c in this directory as a skeleton.
 
 there are still the following problems:
-  * svga 320x200 modes don't work. (but 320x200x8, vga works).
   * pageflipping (in threeDKit) does not work. 
   * no acceleration (is there a program that uses it anyway?).
-  * sometimes does not restore textfont when going back to textmode (or maybe the palette).
-
+  * The clocks for 320x200 modes are _not easy_ for the driver.
+    if you have a problem with those modes, please contact me.
+  * 24 bits modes don't work on AT24.
+  
 */
 
 
 #include <stdlib.h>
 #include <stdio.h>		/* for printf */
 #include <string.h>		/* for memset */
-#include <sys/mman.h>		/* mmap */
 #include <unistd.h>
 #include "vga.h"
 #include "libvga.h"
@@ -177,9 +178,11 @@ static void apm_setregs(const unsigned char regs[], int mode)
     __svgalib_outCR(0x1d,regs[APMREG_SAVE(6)]);
     __svgalib_outCR(0x1e,regs[APMREG_SAVE(7)]);
 
-   k=regs[APMREG_SAVE(10)]+(regs[APMREG_SAVE(11)] << 8)+(regs[APMREG_SAVE(12)] << 16)+(regs[APMREG_SAVE(13)] << 24);
-   outlXR(0xe8,k) ;
-
+   if(apm_chiptype==AT3D) {
+     k=regs[APMREG_SAVE(10)]+(regs[APMREG_SAVE(11)] << 8)+(regs[APMREG_SAVE(12)] << 16)+(regs[APMREG_SAVE(13)] << 24);
+     outlXR(0xe8,k) ;
+   };
+   
    k=regs[APMREG_SAVE(14)]+(regs[APMREG_SAVE(15)] << 8)+(regs[APMREG_SAVE(16)] << 16)+(regs[APMREG_SAVE(17)] << 24);
    outlXR(0xec,k & ~(1 << 7)) ;
    outlXR(0xec,k | (1 << 7)) ;
@@ -199,8 +202,8 @@ static int apm_modeavailable(int mode)
 
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
-/*    if((modeinfo->bitsPerPixel==24)||(0))return 0;
-  */
+    if((modeinfo->bitsPerPixel==24)&&(apm_chiptype==AT24))return 0;
+  
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -434,11 +437,11 @@ static int apm_test(void)
     int     i;
     int	    oldSEQ10;
 
-#if 0
+/* To remove annoying "unused..." warning
     char    id_ap6420[] = "Pro6420";
     char    id_ap6422[] = "Pro6422";
+*/
     char    id_at24[]   = "Pro6424";
-#endif
     char    id_at3d[]   = "ProAT3D"; /* Yeah, the manual could have been 
                                         correct... */
     char    idstring[]  = "       ";
@@ -470,7 +473,8 @@ static int apm_test(void)
     __svgalib_outSR(0x10, oldSEQ10);
     __svgalib_inSR(0);
 
-    if (!memcmp(id_at3d, idstring, 7)) { apm_init(0,0,0) ; return 1; }
+    if (!memcmp(id_at3d, idstring, 7) || !memcmp(id_at24,idstring,7)) 
+      { apm_init(0,0,0) ; return 1; }
     return 0;
 }
 
@@ -565,6 +569,8 @@ DriverSpecs __svgalib_apm_driverspecs =
 
 static int apm_init(int force, int par1, int par2)
 {
+        char    idstring[]  = "       ";
+
     /* Get I/O priviledge */
     if ((getenv("IOPERM") == NULL) && (iopl(3) < 0)) {
 	printf("svgalib(apm): Cannot get I/O permissions.\n");
@@ -582,7 +588,6 @@ static int apm_init(int force, int par1, int par2)
         char    id_at24[]   = "Pro6424";
         char    id_at3d[]   = "ProAT3D"; /* Yeah, the manual could have been 
                                         correct... */
-        char    idstring[]  = "       ";
 
         __svgalib_outSR(0x10, 0x12);
 
@@ -590,37 +595,37 @@ static int apm_init(int force, int par1, int par2)
                idstring[i] = __svgalib_inSR(0x11+i);
         if (!memcmp(id_ap6420, idstring, 7))
            {
-         	apm_chiptype=0;
+         	apm_chiptype=AT6420;
                 return 1;
            }
         else if (!memcmp(id_ap6422, idstring, 7))
            {
-         	apm_chiptype=1;
+         	apm_chiptype=AT6422;
                 return 1;
            }
         else if (!memcmp(id_at24, idstring, 7))
            {
-         	apm_chiptype=2;
-                return 1;
+         	apm_chiptype=AT24;
            }
         else if (!memcmp(id_at3d, idstring, 7))
            {
-           	apm_chiptype=3;            
+           	apm_chiptype=AT3D;            
            }
         /*else return 1;*/
         apm_memory=__svgalib_inSR(0x20)*64-34; /* maybe will support accel some day */
     };
     if (__svgalib_driver_report) {
-	printf("Using AT3d driver, %iKB.\n",apm_memory);
+	printf("Using Alliance driver, %.7s, %iKB.\n",idstring, apm_memory);
     }
 
     apm_xbase= (__svgalib_inSR(0x1f) << 8 ) + __svgalib_inSR(0x1e);
     
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = apm_memory;
-    cardspecs->maxPixelClock4bpp = 75000;	
-    cardspecs->maxPixelClock8bpp = 175500;	
-    cardspecs->maxPixelClock16bpp = 144000;	
+    cardspecs->maxPixelClock4bpp = 75000;
+    cardspecs->maxPixelClock8bpp = 175500;
+    if(apm_chiptype==AT24)cardspecs->maxPixelClock8bpp = 160000;
+    cardspecs->maxPixelClock16bpp = 144000;
     cardspecs->maxPixelClock24bpp = 75000;
     cardspecs->maxPixelClock32bpp = 94500;
     cardspecs->flags = CLOCK_PROGRAMMABLE | INTERLACE_DIVIDE_VERT;
@@ -632,6 +637,11 @@ static int apm_init(int force, int par1, int par2)
     cardspecs->mapHorizontalCrtc = apm_map_horizontal_crtc;
     cardspecs->matchProgrammableClock=apm_match_programmable_clock;
     __svgalib_driverspecs = &__svgalib_apm_driverspecs;
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+    __svgalib_outSR(0x1d,0x193>>2); 
+    __svgalib_linear_mem_base=inb(apm_xbase+3)<<24 ;
+    __svgalib_linear_mem_size=apm_memory*0x400;
     return 0;
 }
 
@@ -648,8 +658,11 @@ comp_lmn(unsigned clock)
   double  fvco_goal;
   double  k, c;
 
-    fmax = 400000.0;
-
+    if(apm_chiptype==AT3D)fmax = 400000.0; else fmax=265000.0;
+/* The XFree86 driver says the max for AT24 is 250000,
+   but there is a much use clock (65MHZ), that is use 260000 fvco,
+   and the VESA bios on the card uses this clock, upped the max */   
+  
   fref = 14318.0;
 
   for (m = 1; m <= 5; m++)
