@@ -17,7 +17,7 @@
 #include "timing.h"
 #include "accel.h"
 
-#define MAX_REGS 4084
+#define MAX_REGS 5000 /* VESA needs a lot of storage space */
 
 typedef struct {
     void (*savepalette)(unsigned char *red, unsigned char *green, unsigned char *blue);
@@ -63,6 +63,7 @@ typedef struct {
 } DriverSpecs;
 
 extern DriverSpecs __svgalib_vga_driverspecs;
+extern DriverSpecs __svgalib_neo_driverspecs;
 extern DriverSpecs __svgalib_cirrus_driverspecs;
 extern DriverSpecs __svgalib_et4000_driverspecs;
 extern DriverSpecs __svgalib_tvga8900_driverspecs;
@@ -82,6 +83,11 @@ extern DriverSpecs __svgalib_nv3_driverspecs;
 extern DriverSpecs __svgalib_et6000_driverspecs;
 extern DriverSpecs __svgalib_vesa_driverspecs;
 extern DriverSpecs __svgalib_mx_driverspecs;
+extern DriverSpecs __svgalib_paradise_driverspecs;
+extern DriverSpecs __svgalib_rage_driverspecs;
+extern DriverSpecs __svgalib_banshee_driverspecs;
+extern DriverSpecs __svgalib_sis_driverspecs;
+extern DriverSpecs __svgalib_i740_driverspecs;
 
 extern DriverSpecs *__svgalib_driverspecs;
 extern DriverSpecs *__svgalib_driverspecslist[];
@@ -117,6 +123,7 @@ extern int __svgalib_critical;
 extern int __svgalib_chipset;
 extern RefreshRange __svgalib_horizsync;
 extern RefreshRange __svgalib_vertrefresh;
+extern int __svgalib_bandwidth;
 extern int __svgalib_grayscale;
 extern int __svgalib_modeinfo_linearset;
 extern const int __svgalib_max_modes;
