@@ -177,9 +177,6 @@
 #include <unistd.h>
 #include <stdarg.h>
 #include <string.h>
-#ifdef USEGLIBC
-#include <sys/io.h>
-#endif
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -686,13 +683,13 @@ static void et4000_lock(void)
 static void et4000_setlinear(int addr)
 {
     et4000_unlock();
-    outb(0x3D4, 0x36);
+    outb(CRT_IC, 0x36);
     if (addr)
-	outb(0x3D5, inb(0x3D5) | 16);	/* enable linear mode */
+	outb(CRT_DC, inb(CRT_DC) | 16);	/* enable linear mode */
     else
-	outb(0x3D5, inb(0x3D5) & ~16);	/* disable linear mode */
-    outb(0x3D4, 0x30);
-    outb(0x3D5, addr);
+	outb(CRT_DC, inb(CRT_DC) & ~16);	/* disable linear mode */
+    outb(CRT_IC, 0x30);
+    outb(CRT_DC, addr);
     et4000_lock();
 }
 
@@ -718,7 +715,7 @@ static int et4000_test(void)
 
     /* test for ET4000 clues */
     if (port_in(0x3cc) & 1)
-	base = 0x3d4;
+	base = CRT_IC;
     else
 	base = 0x3b4;
     port_out(0x33, base);
@@ -784,19 +781,19 @@ static void et4000_setwrpage(int page)
 
 static void et4000_setdisplaystart(int address)
 {
-    outw(0x3d4, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
-    outw(0x3d4, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
+    outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
+    outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
     inb(0x3da);			/* set ATC to addressing mode */
-    outb(0x3c0, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
+    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
-    outb(0x3d4, 0x33);
+    outb(CRT_IC, 0x33);
     if (et4000_chiptype >= CHIP_ET4000W32i)
 	/* write sa18-21 to bit 0-3 */
-	outb(0x3d5, (inb(0x3d5) & 0xf0) | ((address & 0x3c0000) >> 18));
+	outb(CRT_DC, (inb(CRT_DC) & 0xf0) | ((address & ATT_IW000) >> 18));
     else
 	/* write sa18-19 to bit 0-3 */
-	outb(0x3d5, (inb(0x3d5) & 0xfc) | ((address & 0xc0000) >> 18));
+	outb(CRT_DC, (inb(CRT_DC) & 0xfc) | ((address & 0xc0000) >> 18));
 }
 
 
@@ -805,9 +802,9 @@ static void et4000_setdisplaystart(int address)
 
 static void et4000_setlogicalwidth(int width)
 {
-    outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
-    outb(0x3d4, 0x3f);
-    outb(0x3d5, (inb(0x3d5) & 0x7f)
+    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    outb(CRT_IC, 0x3f);
+    outb(CRT_DC, (inb(CRT_DC) & 0x7f)
 	 | ((width & 0x800) >> 5));	/* write lw12 to bit 7 */
 }
 
@@ -1065,8 +1062,8 @@ static int et4000_init(int force, int par1, int par2)
     if (force)
 	et4000_memory = par1;
     else {
-	outb(0x3d4, 0x37);
-	value = inb(0x3d5);
+	outb(CRT_IC, 0x37);
+	value = inb(CRT_DC);
 	et4000_memory = (value & 0x08) ? 256 : 64;
 	switch (value & 3) {
 	case 2:
@@ -1078,8 +1075,8 @@ static int et4000_init(int force, int par1, int par2)
 	}
 	if (value & 0x80)
 	    et4000_memory *= 2;
-	outb(0x3d4, 0x32);
-	if (inb(0x3d5) & 0x80) {
+	outb(CRT_IC, 0x32);
+	if (inb(CRT_DC) & 0x80) {
 	    /* Interleaved memory on ET4000/W32i/p: */
 	    /* Multiply by 2. */
 	    et4000_memory *= 2;
@@ -1137,6 +1134,11 @@ static int et4000_init(int force, int par1, int par2)
     }
     __svgalib_driverspecs = &__svgalib_et4000_driverspecs;
 
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+/*    __svgalib_linear_mem_base=LINEARBASE;
+    __svgalib_linear_mem_size=nv3_memory*0x400; */
+
     pos_ext_settings = 0;
     if (et4000_dac & (8 | 16 | 32 | 64))
 	pos_ext_settings = VGA_CLUT8;
