@@ -49,7 +49,7 @@ static struct termios oldkbdtermios, newkbdtermios;
 static int oldkbmode;
 /* vga.c needs to check that: */
 int __svgalib_kbd_fd = -1; /* nowadays merely used as a flag */
-static int c_state, ctrl_state, alt_state, functionkey_state;
+static int c_state, ctrl_state, alt_state, functionkey_state, win_state;
 static int translatemode = 0;
 static unsigned char state[NR_KEYS];	/* NR_KEYS is defined in linux/keyboard.h */
 static int keymap[NR_KEYS];
@@ -112,7 +112,7 @@ static int fme_used = 0;
 
 static int expand_events(int n) {
     struct FakeMouseEvent *newptr = 0;
-//    int n;
+/*    int n; */
 
   if ( ! fake_mouse_events) {
 #ifdef DEBUG_KEYBOARD
@@ -477,6 +477,11 @@ static int keyboard_getevents(int wait)
 		alt_state = is_pressed;
 		break;
 
+	  case SCANCODE_LEFTWIN:
+	  case SCANCODE_RIGHTWIN:
+             	win_state = is_pressed;
+		break;
+
 	  case SCANCODE_F1...SCANCODE_F10:
 		functionkey_state = (is_pressed)  ?
 		  functionkey_state |   1 << (scancode - SCANCODE_F1)   :
@@ -569,14 +574,15 @@ static int keyboard_getevents(int wait)
 
 	if (ctrl_state && c_state && !(translatemode & DONT_CATCH_CTRLC))
 	    raise(SIGINT);
-	if (alt_state && functionkey_state) {
+	if ((alt_state && functionkey_state)||(win_state && functionkey_state)) {
 	    /* VT switch. */
 	    /* *** what about F11 & F12? */
 	    int j, vt = 0;
 	    struct vt_stat vts;
-	    for (j = 0; j < 10; j++)
+	    for (j = 0; j < 12; j++)
 		if (functionkey_state & (1 << j)) {
 		    vt = j + 1;
+                    if(win_state)j+=12;
 		    break;
 		}
 
