@@ -8,6 +8,9 @@
 #define _LIBVGA_H
 
 #include <string.h>
+# if defined __GLIBC__ && __GLIBC__ >= 2
+#include <sys/io.h>
+#endif
 
 /* --------------------- Macro definitions shared by library modules */
 
@@ -141,6 +144,11 @@ struct info {
 extern int __svgalib_CRT_I;		/* current CRT index register address */
 extern int __svgalib_CRT_D;		/* current CRT data register address */
 extern int __svgalib_IS1_R;		/* current input status register address */
+extern unsigned char * BANKED_MEM_POINTER, * LINEAR_MEM_POINTER, * MMIO_POINTER;
+extern unsigned long int __svgalib_banked_mem_base, __svgalib_banked_mem_size;
+extern unsigned long int __svgalib_mmio_base, __svgalib_mmio_size;
+extern unsigned long int __svgalib_linear_mem_base, __svgalib_linear_mem_size;
+extern unsigned long int __svgalib_mmio_base, __svgalib_mmio_size;
 extern struct info CI;		/* current video parameters */
 extern int COL;			/* current color            */
 extern int CM;			/* current video mode       */
@@ -154,6 +162,9 @@ extern unsigned char *SM;	/* sparse graphics memory frame */
 extern int MODEX;		/* TRUE after vga_setmodeX() */
 extern int MODEFLAGS;		/* copy of flags of current modeinfo->flags */
 
+#ifdef BACKGROUND
+extern int __svgalib_virtual_mem_fd;
+#endif
 extern int __svgalib_mem_fd;
 extern int __svgalib_tty_fd;
 extern int __svgalib_nosigint;
@@ -164,6 +175,7 @@ extern int __svgalib_runinbackground;
 extern unsigned char __svgalib_novga;
 extern unsigned char __svgalib_textprog;
 extern unsigned char __svgalib_secondary;
+extern unsigned char __svgalib_novccontrol;
 extern unsigned char __svgalib_m_ignore_dx;
 extern unsigned char __svgalib_m_ignore_dy;
 extern unsigned char __svgalib_m_ignore_dz;
@@ -172,6 +184,13 @@ extern char *__joystick_devicenames[4];
 
 /* --------------------- Function definitions shared by library modules */
 
+extern int (*__svgalib_inmisc)(void);
+extern void (*__svgalib_outmisc)(int);
+extern int (*__svgalib_incrtc)(int);
+extern void (*__svgalib_outcrtc)(int,int);
+extern int (*__svgalib_inseq)(int);
+extern void (*__svgalib_outseq)(int,int);
+
 extern int __svgalib_setregs(const unsigned char *regs);
 extern int __svgalib_saveregs(unsigned char *regs);
 extern void __svgalib_dumpregs(const unsigned char regs[], int n);
@@ -189,7 +208,11 @@ extern void (*__svgalib_keyboard_eventhandler) (int, int);
 extern void __joystick_flip_vc(int acquire);
 extern char *__svgalib_TextProg_argv[16]; /* should be enough */
 extern char *__svgalib_TextProg;
+extern int __svgalib_VESA_savebitmap;
+extern int __svgalib_VESA_textmode;
 extern unsigned char __svgalib_vesatext;
+extern int __svgalib_mapkeyname(const char *keyname);
+extern void __svgalib_mouse_update_keymap(void);
 
 #if 0 
 /* remove this part ? */
@@ -331,3 +354,15 @@ extern unsigned char *__svgalib_give_graph_blue(void);
 #endif
 
 #define zero_sa_mask(maskptr) memset(maskptr, 0, sizeof(sigset_t))
+
+#if 0
+
+#define SVGALIB_ACQUIRE_SIG SIGUSR2
+#define SVGALIB_RELEASE_SIG SIGUSR1
+
+#else
+
+#define SVGALIB_ACQUIRE_SIG SIGUNUSED
+#define SVGALIB_RELEASE_SIG SIGPROF
+
+#endif
