@@ -13,6 +13,8 @@
  * Added mouse acceleration 3/97 - Mike Chapman mike@paranoia.com
  * Added Intellimouse support 5/98 - Brion Vibber brion@pobox.com
  * Totally customisable mouse behaviour 28 Mar 1998 - by 101; 101@kempelen.inf.bme.hu
+ * Added rx-axis support for IntelliMouse wheel and improved fake keyboard
+ *  event scancode setting 7/98 - Brion
  */
 
 /* This file is included by mouse.c. */
@@ -31,6 +33,7 @@
 
 /* #define DEBUG */
 /* #define DEBUG_ACCEL */
+/* #define DEBUG_WHEEL */
 
 static int    m_type;
 static int    m_baud = 1200;		/* Should be 1200. */
@@ -58,9 +61,13 @@ static float  m_accel_power  = DEFAULT_ACCEL_POWER;
 static float  m_accel_offset = DEFAULT_ACCEL_OFFSET;
 static int    m_force	     = 0;			/* Don't force parameters */
 
-static int m_fake_kbd_events = 0;                       /* Create fake keyboard events for turning the wheel (for IntelliMouse & co) */
+static int m_wheel_steps = DEFAULT_WHEEL_STEPS;         /* Number of steps that make up a full turn of the wheel (for IntelliMouse & co) */
+static int m_wheel_delta = DEFAULT_WHEEL_DELTA;         /* Amount to change rotation about the X axis when wheel is turned */
+static int m_fake_kbd_events = 0;                       /* Create fake keyboard events for turning the wheel */
 static int m_fake_upscancode = 0;                       /* Scan code to produce for turning up */
 static int m_fake_downscancode = 0;                     /* Scan code to produce for turning down */
+static char *m_fake_upkeyname = NULL;                   /* Symbolic key name from which scancode is determined */
+static char *m_fake_downkeyname = NULL;
 
 static const unsigned short cflag[] = {
     (CS7 | CREAD | CLOCAL | HUPCL),			/* MicroSoft */
@@ -156,57 +163,7 @@ static char *mouse_config_options[] = {
   "mouse_accel_type", "mouse_accel_mult", "mouse_accel_thresh",
   "mouse_accel_power", "mouse_accel_maxdelta", "mouse_maxdelta",
   "mouse_accel_offset", "mouse_override", "mouse_force",
-  "mouse_fake_kbd_event", NULL
-};
-
-/* This is for specifying scancodes in the config file for fake kbd events */
-/* We should be able to move this to the generic section and use it for the
- keyboard's fake mouse events also */
-
-static struct {
-    char *name;
-    int code;
-} mouse_scancodes[] = {
-    {"ESCAPE",1},{"1",2},{"2",3},{"3",4},{"4",5},{"5",6},{"6",7},{"7",8},
-    {"8",9},{"9",10},{"0",11},{"MINUS",12},{"EQUAL",13},{"BACKSPACE",14},
-    
-    {"TAB",15},{"Q",16},{"W",17},{"E",18},{"R",19},{"T",20},{"Y",21},{"U",22},
-    {"I",23},{"O",24},{"P",25},{"BRACKET_LEFT",26},{"BRACKET_RIGHT",27},
-    {"ENTER",28},
-
-    {"LEFTCONTROL",29},{"A",30},{"S",31},{"D",32},{"F",33},{"G",34},{"H",35},
-    {"J",36},{"K",37},{"L",38},{"SEMICOLON",39},{"APOSTROPHE",40},{"GRAVE",41},
-
-    {"LEFTSHIFT",42},{"BACKSLASH",43},{"Z",44},{"X",45},{"C",46},{"V",47},
-    {"B",48},{"N",49},{"M",50},{"COMMA",51},{"PERIOD",52},{"SLASH",53},
-    {"RIGHTSHIFT",54},
-
-    {"KEYPADMULTIPLY",55},{"LEFTALT",56},{"SPACE",57},{"CAPSLOCK",58},
-
-    {"F1",59},{"F2",60},{"F3",61},{"F4",62},{"F5",63},{"F6",64},{"F7",65},
-    {"F8",66},{"F9",67},{"F10",68},
-
-    {"NUMLOCK",69},{"SCROLLLOCK",70},
-
-    {"KEYPAD7",71},{"CURSORUPLEFT",71},{"KEYPAD8",72},{"CURSORUP",72},
-    {"KEYPAD9",73},{"CURSORUPRIGHT",73},{"KEYPADMINUS",74},{"KEYPAD4",75},
-    {"CURSORLEFT",75},{"KEYPAD5",76},{"KEYPAD6",77},{"CURSORRIGHT",77},
-    {"KEYPADPLUS",78},{"KEYPAD1",79},{"CURSORDOWNLEFT",79},{"KEYPAD2",80},
-    {"CURSORDOWN",80},{"KEYPAD3",81},{"CURSORDOWNRIGHT",81},{"KEYPAD0",82},
-    {"KEYPADPERIOD",83},
-
-    {"LESS",86}, /* What is this? */
-
-    {"F11",87},{"F12",88},
-
-    {"KEYPADENTER",96},{"RIGHTCONTROL",97},{"CONTROL",97},{"KEYPADDIVIDE",98},
-    {"PRINTSCREEN",99},{"RIGHTALT",100},{"BREAK",101},{"BREAK_ALTERNATIVE",119},
-
-    {"HOME",102},{"CURSORBLOCKUP",103},{"PAGEUP",104},{"CURSORBLOCKLEFT",105},
-    {"CURSORBLOCKRIGHT",106},{"END",107},{"CURSORBLOCKDOWN",108},{"PAGEDOWN",109},
-    {"INSERT",110},{"REMOVE",111},
-
-    {NULL,0}
+  "mouse_fake_kbd_event", "mouse_wheel_steps", NULL
 };
 
 static char* mouse_process_option(int option, int mode) {
@@ -221,6 +178,7 @@ static char* mouse_process_option(int option, int mode) {
    switch (option) {
     float *fptr;
     int   *iptr;
+    char  **cptr;
 
      case 0: /* mouse_accel_type */
 	if ( (ptr = strtok(NULL, " ")) == 0 ) {
@@ -317,42 +275,55 @@ static char* mouse_process_option(int option, int mode) {
             Turn them off temporarily in case the params are bad */
          m_fake_kbd_events = 0;
 
+#ifdef DEBUG_WHEEL
+         fprintf(stderr, " Setting up fake keyboard events");
+#endif
+
          /* Parse the config for the scancodes */
+         cptr = &m_fake_upkeyname;
+         iptr = &m_fake_upscancode;
+         while(cptr) {
+             if ( (ptr = strtok(NULL, " ")) == 0 )
+                 goto param_needed;
 
-         /* up */
-         if ( (ptr = strtok(NULL, " ")) == 0 ) {
-             goto param_needed;
-         } else {
-             /* Go through the list of scancodes */
-             int i;
-             for(i = 0; mouse_scancodes[i].name; i++)
-                 if(strcasecmp(ptr,mouse_scancodes[i].name) == 0)
-                     break;
-             if((m_fake_upscancode = mouse_scancodes[i].code) == 0) {
-                 fprintf(stderr,"svgalib: mouse-config: illegal scancode \'%s\' for mouse_fake_kbd_event\n",ptr);
-                 return ptr;
-             }
+#ifdef DEBUG_WHEEL
+             fprintf(stderr, "; keyname %s", ptr);
+#endif
+
+             /* Keep the name in case the keyboard gets remapped later */
+             if(*cptr)
+                 free(*cptr);
+             *cptr = (char *)malloc(strlen(ptr) + 1);
+             strcpy(*cptr, ptr);
+
+             /* Determine the scancode */
+             *iptr = __svgalib_mapkeyname(*cptr);
+
+#ifdef DEBUG_WHEEL
+             fprintf(stderr, " = scancode %d", *iptr);
+#endif
+
+             /* Move on to the next */
+             iptr = ((cptr == &m_fake_upkeyname)?(&m_fake_downscancode):NULL);
+             cptr = ((cptr == &m_fake_upkeyname)?(&m_fake_downkeyname):NULL);
          }
 
-         /* down */
-         if ( (ptr = strtok(NULL, " ")) == 0 ) {
-             goto param_needed;
-         } else {
-             /* Go through the list of scancodes */
-             int i;
-             for(i = 0; mouse_scancodes[i].name; i++)
-                 if(strcasecmp(ptr,mouse_scancodes[i].name) == 0)
-                     break;
-             if(!(m_fake_downscancode = mouse_scancodes[i].code)) {
-                 fprintf(stderr,"svgalib: mouse-config: illegal scancode \'%s\' for mouse_fake_kbd_event\n",ptr);
-                 return ptr;
-             }
-         }
+#ifdef DEBUG_WHEEL
+         fprintf(stderr, "; done.\n");
+#endif
 
          /* Now that we've survived all that, turn on fake events */
          m_fake_kbd_events = 1;
 
-       break;
+         break;
+
+   case 10: /* mouse_wheel_steps */
+       /* Set the number of steps that make up a complete turn of
+          the wheel on an IntelliMouse or other wheel mouse. Zero
+          disables X-axis rotation but not fake keyboard events. */
+	iptr = &m_wheel_steps;
+	goto process_int;
+
    }
    return strtok(NULL, " ");
 }
@@ -451,7 +422,16 @@ static int ms_init(void)
     if (fcntl(1,F_GETFD) < 0) open("/dev/null", O_WRONLY);
     if (fcntl(2,F_GETFD) < 0) open("/dev/null", O_WRONLY);
 
+    /* Set the proper wheel delta */
+    if(m_wheel_steps)
+        m_wheel_delta = (360 / m_wheel_steps);
+    else
+        m_wheel_delta = 0;
+
     /* Added O_NDELAY here. */
+/*
+    if(__svgalib_mouse_fd!=-1)close(__svgalib_mouse_fd);
+*/
     if ((__svgalib_mouse_fd = open(m_dev, O_RDWR | O_NDELAY )) < 0)
 	return -1;
 
@@ -529,6 +509,48 @@ static int ms_init(void)
 	else
 	    write(__svgalib_mouse_fd, "N", 1);
     }
+
+    /* Set the mouse caps */
+    mouse_caps.key = MOUSE_GOTCAPS;
+
+    /* Zero out first to be sure... */
+    mouse_caps.buttons = mouse_caps.axes = mouse_caps.info =
+        mouse_caps.reserved0 = mouse_caps.reserved1 = 0;
+
+    switch(m_type) {
+    case MOUSE_SPACEBALL:
+        /* 6 axes, many buttons */
+        mouse_caps.buttons =
+            MOUSE_LEFTBUTTON | MOUSE_MIDDLEBUTTON | MOUSE_RIGHTBUTTON |
+            MOUSE_FOURTHBUTTON | MOUSE_FIFTHBUTTON | MOUSE_SIXTHBUTTON |
+            MOUSE_RESETBUTTON;
+        mouse_caps.axes = MOUSE_6DIM;
+        break;
+        
+    case MOUSE_INTELLIMOUSE:
+    case MOUSE_IMPS2:
+        /* X, Y, RX (wheel), 3 buttons, wheel */
+        mouse_caps.axes = MOUSE_RXDIM;
+        mouse_caps.info = MOUSE_INFO_WHEEL;
+
+    case MOUSE_LOGIMAN:		/* Some TrackMen have 4 buttons */
+	mouse_caps.buttons = MOUSE_FOURTHBUTTON;
+
+    case MOUSE_MOUSESYSTEMS:
+    case MOUSE_MMSERIES:
+    case MOUSE_LOGITECH:
+    case MOUSE_BUSMOUSE:
+    case MOUSE_PS2:
+    case MOUSE_GPM:
+        /* Any of these _can_ have 3 buttons, but may not */
+        mouse_caps.buttons |= MOUSE_MIDDLEBUTTON;
+
+    case MOUSE_MICROSOFT: /* Two buttons only */
+        mouse_caps.buttons |= MOUSE_LEFTBUTTON | MOUSE_RIGHTBUTTON;
+        mouse_caps.axes |= MOUSE_2DIM;
+        break;
+    }
+    
     return 0;
 }
 
@@ -568,7 +590,7 @@ static int get_ms_event(int wait) {
     int nu_packets = 0;
     char event_handled = 0;
     int bytesread;
-    int i;
+    int i, wheel;
 /*  int but; */
     static int but = 0;		/* static is hack for MouseMan */
     static int mouse_orientation = 0;
@@ -621,9 +643,9 @@ static int get_ms_event(int wait) {
     if ((m_type == MOUSE_LOGIMAN) &&
 	((nu_bytes - i) >= 1) &&
 	((buf[i] & proto[m_type][0]) != proto[m_type][1]) &&
-	((char) (buf[i] & ~0x23) == 0)) {
+	((char) (buf[i] & ~0x33) == 0)) { /* s/23/33/, for 4-but trackman */
 	/* Hack-o-matic, stolen from xf86_Mouse.c */
-	but = ((buf[i] & 0x20) >> 4) | (but & 0x05);
+	but = ((buf[i] & 0x20) >> 4) | ((buf[i] & 0x10) >> 1) | (but & 0x05);
 	__svgalib_mouse_eventhandler(but, 0, 0, 0, 0, 0, 0);
 	event_handled++;
 	i++;
@@ -696,7 +718,7 @@ static int get_ms_event(int wait) {
     case MOUSE_MICROSOFT:	/* Microsoft */
     case MOUSE_LOGIMAN:	/* MouseMan / TrackMan */
     default:
-	but = (but & 2) | ((buf[i] & 0x20) >> 3) | ((buf[i] & 0x10) >> 4);
+	but = (but & 0x0A) | ((buf[i] & 0x20) >> 3) | ((buf[i] & 0x10) >> 4);
 	dx = (char) (((buf[i] & 0x03) << 6) | (buf[i + 1] & 0x3F));
 	dy = (char) (((buf[i] & 0x0C) << 4) | (buf[i + 2] & 0x3F));
         break;
@@ -708,13 +730,36 @@ static int get_ms_event(int wait) {
         dx = (char) (((buf[i] & 0x03) << 6) | (buf[i + 1] & 0x3F));
         dy = (char) (((buf[i] & 0x0C) << 4) | (buf[i + 2] & 0x3F));
 
-        /* Did we turn the wheel and are fake keyboard events enabled? */
-        if((buf[i + 3] & 0x0f) && m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
-            /* Fake keypresses... */
-            int key = (((buf[i + 3] & 0x0f) > 7) ? m_fake_upscancode : m_fake_downscancode);
-            __svgalib_keyboard_eventhandler(key, 1); /* press */
-            /* todo: delay */
-            __svgalib_keyboard_eventhandler(key, 0); /* release */
+        /* Did we turn the wheel? */
+        if((wheel = buf[i + 3] & 0x0f) != 0) {
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, " Wheel turned (0x%02x)", wheel);
+#endif
+            /* RX-axis */
+            if(m_wheel_delta) {
+                drx = ((wheel > 7) ? (-m_wheel_delta) : m_wheel_delta);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; RX axis delta = %d", drx);
+#endif
+            }
+
+            /* Are fake keyboard events enabled? */
+            if(m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
+                /* Fake keypresses... */
+                int key = ((wheel > 7) ? m_fake_upscancode : m_fake_downscancode);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; pressing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 1); /* press */
+                /* todo: delay */
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, ", releasing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 0); /* release */
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, ".\n");
+#endif
+            }
         }
         break;
     case MOUSE_MOUSESYSTEMS:	/* Mouse Systems Corp */
@@ -750,13 +795,36 @@ static int get_ms_event(int wait) {
         dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
         dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
 
-        /* Did we turn the wheel and are fake keyboard events enabled? */
-        if(buf[i + 3] && m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
-            /* Fake keypresses... */
-            int key = (((buf[i + 3]) > 0x7f) ? m_fake_upscancode : m_fake_downscancode);
-            __svgalib_keyboard_eventhandler(key, 1); /* press */
-            /* todo: delay */
-            __svgalib_keyboard_eventhandler(key, 0); /* release */
+        /* Did we turn the wheel? */
+        if((wheel = buf[i + 3]) != 0) {
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, " Wheel turned (0x%02x)", wheel);
+#endif
+            /* RX-axis */
+            if(m_wheel_delta) {
+                drx = ((wheel > 0x7f) ? (-m_wheel_delta) : m_wheel_delta);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; RX axis delta = %d", drx);
+#endif
+            }
+
+            /* Are fake keyboard events enabled? */
+            if(m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
+                /* Fake keypresses... */
+                int key = ((wheel > 0x7f) ? m_fake_upscancode : m_fake_downscancode);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; pressing scancode %d", key);
+#endif
+                 __svgalib_keyboard_eventhandler(key, 1); /* press */
+                /* todo: delay */
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, ", releasing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 0); /* release */
+            }
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, ".\n");
+#endif
         }
         break;
     case MOUSE_SPACEBALL:
@@ -923,3 +991,21 @@ static int get_ms_event(int wait) {
 
     goto handle_packets;
 }
+
+void __svgalib_mouse_update_keymap(void)
+{
+    /*
+     The keyboard has been remapped, and we should accordingly remap any
+     symbolically-specified scancodes for fake keyboard events.
+     */
+#ifdef DEBUG_WHEEL
+    fprintf(stderr, " Remapping fake keyboard event scancodes... ");
+#endif
+    if(m_fake_upkeyname)
+        m_fake_upscancode = __svgalib_mapkeyname(m_fake_upkeyname);
+    if(m_fake_downkeyname)
+        m_fake_downscancode = __svgalib_mapkeyname(m_fake_downkeyname);
+#ifdef DEBUG_WHEEL
+    fprintf(stderr, "done.\n");
+#endif
+}
