@@ -11,14 +11,13 @@ there are still the following problems:
   * no 24bit modes. (maybe the hardware does not support)
   * pageflipping (in threeDKit) does not work. 
   * no acceleration (is there a program that uses it anyway?).
-  
 */
 
 
 #include <stdlib.h>
 #include <stdio.h>		/* for printf */
 #include <string.h>		/* for memset */
-#include <sys/mman.h>		/* mmap */
+#include <sys/mman.h>		
 #include <fcntl.h>
 #include <math.h>
 #include <unistd.h>
@@ -38,7 +37,6 @@ there are still the following problems:
 
 #define NV3REG_SAVE(i) (VGA_TOTAL_REGS+i)
 #define NV3_TOTAL_REGS (VGA_TOTAL_REGS + 24 + 33 )
-#define PLL_INPUT_FREQ 13500 
 #define M_MIN 7
 #define M_MAX 13
 
@@ -51,6 +49,7 @@ there are still the following problems:
 
 static volatile unsigned  *nvPFBPort;
 static volatile unsigned  *nvPRAMDACPort;
+static volatile unsigned  *nvPEXTDEVPort;
 
 static int nv3_init(int, int, int);
 static void nv3_unlock(void);
@@ -60,18 +59,18 @@ void __svgalib_nv3accel_init(AccelSpecs * accelspecs, int bpp, int width_in_pixe
 static int nv3_memory,nv3_chiptype;
 static int nv3_is_linear;
 static CardSpecs *cardspecs;
+static int PLL_INPUT_FREQ;
 
 static unsigned long MMIOBASE, LINEARBASE;
 
-
 enum {
-   NV3_chipset = 0
+   Riva128 = 0, RivaTNT, GEFORCE
 };
 
 static void nv3_setpage(int page)
 {
-__svgalib_outCR(0x1d,page << 1);
-__svgalib_outCR(0x1e,page << 1);
+__svgalib_outcrtc(0x1d,page << 1);
+__svgalib_outcrtc(0x1e,page << 1);
 }
 
 static int __svgalib_nv3_inlinearmode(void)
@@ -137,38 +136,37 @@ static int nv3_saveregs(unsigned char regs[])
    regs[NV3REG_SAVE(22)] = (k >> 16) & 0xff ;
    regs[NV3REG_SAVE(23)] = ( k >> 24 ) & 0xff ;
    
-for(i=0x18;i<0x38;i++)regs[84+i-0x18]=__svgalib_inCR(i);
+   for(i=0x18;i<0x38;i++)regs[84+i-0x18]=__svgalib_incrtc(i);
    
-    return NV3_TOTAL_REGS - VGA_TOTAL_REGS;
+   return NV3_TOTAL_REGS - VGA_TOTAL_REGS;
 }
 
 /* Set chipset-specific registers */
 
 static void nv3_setregs(const unsigned char regs[], int mode)
-{  unsigned long k ; 
+{   unsigned long k ; 
     nv3_unlock();		/* May be locked again by other programs (eg. X) */
 
-  PCRTC_Write(REPAINT0,regs[NV3REG_SAVE(0)]);
-  PCRTC_Write(REPAINT1,regs[NV3REG_SAVE(1)]);
-  PCRTC_Write(EXTRA,regs[NV3REG_SAVE(2)]);
-  PCRTC_Write(PIXEL,regs[NV3REG_SAVE(3)]);
-  PCRTC_Write(HORIZ_EXTRA,regs[NV3REG_SAVE(4)]); 
-  PCRTC_Write(FIFO_CONTROL,regs[NV3REG_SAVE(5)]); 
+   PCRTC_Write(REPAINT0,regs[NV3REG_SAVE(0)]);
+   PCRTC_Write(REPAINT1,regs[NV3REG_SAVE(1)]);
+   PCRTC_Write(EXTRA,regs[NV3REG_SAVE(2)]);
+   PCRTC_Write(PIXEL,regs[NV3REG_SAVE(3)]);
+   PCRTC_Write(HORIZ_EXTRA,regs[NV3REG_SAVE(4)]); 
+   PCRTC_Write(FIFO_CONTROL,regs[NV3REG_SAVE(5)]); 
+/*  PCRTC_Write(FIFO,regs[NV3REG_SAVE(6)]); */
 
-
-__svgalib_outCR(0x1c,regs[88]); /* this enables banking at 0xa0000 */
-__svgalib_outCR(0x31,0); /* disable cursor */
+   __svgalib_outcrtc(0x1c,regs[88]); /* this enables banking at 0xa0000 */
+   __svgalib_outcrtc(0x31,0); /* disable cursor */
 
    k=regs[NV3REG_SAVE(8)]+(regs[NV3REG_SAVE(9)] << 8)+(regs[NV3REG_SAVE(10)] << 16)+(regs[NV3REG_SAVE(11)] << 24); 
-  PFB_Write(CONFIG_0,k);
+          PFB_Write(CONFIG_0,k);
 
    k=regs[NV3REG_SAVE(12)]+(regs[NV3REG_SAVE(13)] << 8)+(regs[NV3REG_SAVE(14)] << 16)+(regs[NV3REG_SAVE(15)] << 24);
-  PRAMDAC_Write(VPLL_COEFF,k);
+          PRAMDAC_Write(VPLL_COEFF,k);
    k=regs[NV3REG_SAVE(16)]+(regs[NV3REG_SAVE(17)] << 8)+(regs[NV3REG_SAVE(18)] << 16)+(regs[NV3REG_SAVE(19)] << 24);
-  PRAMDAC_Write(PLL_COEFF_SELECT,k);
+          PRAMDAC_Write(PLL_COEFF_SELECT,k);
    k=regs[NV3REG_SAVE(20)]+(regs[NV3REG_SAVE(21)] << 8)+(regs[NV3REG_SAVE(22)] << 16)+(regs[NV3REG_SAVE(23)] << 24);
-  PRAMDAC_Write(GENERAL_CONTROL,k);
-
+          PRAMDAC_Write(GENERAL_CONTROL,k);
 }
 
 
@@ -273,10 +271,8 @@ static int CalculateCRTC(ModeTiming *mode, ModeInfo *modeinfo, unsigned char *mo
   return 1;
 }
 
-
 /* Set a mode */
 
-
 /* Local, called by nv3_setmode(). */
 
 static void nv3_initializemode(unsigned char *moderegs,
@@ -287,10 +283,10 @@ static void nv3_initializemode(unsigned char *moderegs,
   float clockOut;
   int pixelDepth;
   
-    nv3_saveregs(moderegs);
-    __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
-  if(!NV3ClockSelect(clockIn,&clockOut,&m,&n,&p)) {return ;}
-  k=PRAMDAC_Val(VPLL_COEFF_NDIV,n) | PRAMDAC_Val(VPLL_COEFF_MDIV,m) | PRAMDAC_Val(VPLL_COEFF_PDIV,p);
+   nv3_saveregs(moderegs);
+   __svgalib_setup_VGA_registers(moderegs, modetiming, modeinfo);
+   if(!NV3ClockSelect(clockIn,&clockOut,&m,&n,&p)) {return ;}
+   k=PRAMDAC_Val(VPLL_COEFF_NDIV,n) | PRAMDAC_Val(VPLL_COEFF_MDIV,m) | PRAMDAC_Val(VPLL_COEFF_PDIV,p);
    moderegs[NV3REG_SAVE(12)] = k&0xff;
    moderegs[NV3REG_SAVE(13)] = (k >> 8)&0xff;
    moderegs[NV3REG_SAVE(14)] = (k >> 16) & 0xff ;
@@ -302,11 +298,10 @@ static void nv3_initializemode(unsigned char *moderegs,
     PCRTC_Val(REPAINT1_LARGE_SCREEN,modetiming->CrtcHDisplay<1280) |
     PCRTC_Def(REPAINT1_PALETTE_WIDTH,6BITS);
 
-  /* Need to figure out what the algorithm to set these are */
-  moderegs[NV3REG_SAVE(5)]=0x83;/*PCRTC_Def(FIFO_CONTROL_BURST_LENGTH,64);*/
-  moderegs[NV3REG_SAVE(6)]=0x22;/*PCRTC_Val(FIFO_WATERMARK,256>>3)|
-                     PCRTC_Val(FIFO_RESET,1);*/
-
+/* The new xfree driver (from nVidia) calculates those in some
+   twisted way, but I leave it for now */
+  moderegs[NV3REG_SAVE(5)]=0x83;
+  moderegs[NV3REG_SAVE(6)]=0x22;
 
   /* PixelFormat controls how many bits per pixel. 
    * There is another register in the 
@@ -323,16 +318,24 @@ static void nv3_initializemode(unsigned char *moderegs,
      PRAMDAC_Def(GENERAL_CONTROL_BPC,6BITS)|  
      PRAMDAC_Def(GENERAL_CONTROL_VGA_STATE,SEL); /* Not sure about this */
 
-if(modeinfo->bitsPerPixel>8)k|=PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS);
+   if(modeinfo->bitsPerPixel>8)k|=PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS);
 
    moderegs[NV3REG_SAVE(20)] = k&0xff;
    moderegs[NV3REG_SAVE(21)] = (k >> 8)&0xff;
    moderegs[NV3REG_SAVE(22)] = (k >> 16) & 0xff ;
    moderegs[NV3REG_SAVE(23)] = ( k >> 24 ) & 0xff ;
 
-  k=PRAMDAC_Def(PLL_COEFF_SELECT_MPLL_SOURCE,PROG)|
+   switch(nv3_chiptype){
+      case Riva128:
+          k=PRAMDAC_Def(PLL_COEFF_SELECT_MPLL_SOURCE,PROG)|
                             PRAMDAC_Def(PLL_COEFF_SELECT_VPLL_SOURCE,PROG)|
                             PRAMDAC_Def(PLL_COEFF_SELECT_VCLK_RATIO,DB2);
+          break;
+      case RivaTNT:
+          k=0x10000700;
+          break;
+   };
+   
    moderegs[NV3REG_SAVE(16)] = k&0xff;
    moderegs[NV3REG_SAVE(17)] = (k >> 8)&0xff;
    moderegs[NV3REG_SAVE(18)] = (k >> 16) & 0xff ;
@@ -342,7 +345,7 @@ if(modeinfo->bitsPerPixel>8)k|=PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS);
    * give some significant performance gains. Will perhaps experiment 
    * later on with this stuff!  
    */
-  k=
+   k=
       PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
       PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
       PFB_Def(CONFIG_0_TILING,ENABLED); 
@@ -351,18 +354,16 @@ if(modeinfo->bitsPerPixel>8)k|=PRAMDAC_Def(GENERAL_CONTROL_BPC,8BITS);
    moderegs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
    moderegs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
 
-moderegs[88]=28;
+  moderegs[88]=28;
 
-if(modeinfo->greenWeight==5){moderegs[NV3REG_SAVE(21)]&=0xef;}; 
+  if(modeinfo->greenWeight==5){moderegs[NV3REG_SAVE(21)]&=0xef;}; 
 
-nv3_is_linear=0;
+  nv3_is_linear=0;
 
 return ;
-
 }
 
 
-
 static int nv3_setmode(int mode, int prv_mode)
 {
     unsigned char *moderegs;
@@ -371,6 +372,7 @@ static int nv3_setmode(int mode, int prv_mode)
     int i;
     if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
 	|| mode == G720x348x2) {
+        if(nv3_chiptype==Riva128)PRAMDAC_Write(PLL_COEFF_SELECT,0x00000100);
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
 
@@ -407,7 +409,14 @@ static int nv3_setmode(int mode, int prv_mode)
 
 static void nv3_unlock(void)
 {
-__svgalib_outSR(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC);    
+    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
+    __svgalib_outseq(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC);    
+}
+
+static void nv4_unlock(void)
+{
+    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
+    __svgalib_outcrtc(0x1f, UNLOCK_EXT_MAGIC);
 }
 
 
@@ -416,11 +425,10 @@ __svgalib_outSR(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC);
 
 static void nv3_lock(void)
 {
-__svgalib_outSR(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC+1);    
+__svgalib_outseq(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC+1);    
 
 }
 
-
 /* Indentify chipset, initialize and return non-zero if detected */
 
 static int nv3_test(void)
@@ -435,10 +443,33 @@ static int nv3_test(void)
        exit(1);
      }
    }
-   found=__svgalib_pci_find_vendor_vga(0x12d2,buf);
+   found=__svgalib_pci_find_vendor_vga(0x12d2,buf,0);
+   if (found) {
+      found=__svgalib_pci_find_vendor_vga(0x10de,buf,0);
+      if (_ioperm) iopl(0);
+      if(found)return 0;
+   };
    if (_ioperm) iopl(0);
-   if (found)return 0;
-   if (((buf[0]>>16)&0xffff)!=0x18)return 0;
+   if ((((buf[0]>>16)&0xffff)!=0x18)&&
+       (((buf[0]>>16)&0xffff)!=0x20)&&
+       (((buf[0]>>16)&0xffff)!=0x29)&&
+       (((buf[0]>>16)&0xffff)!=0x2C)&&
+       (((buf[0]>>16)&0xffff)!=0x2D)&&
+       (((buf[0]>>16)&0xffff)!=0xA0)&&
+       (((buf[0]>>16)&0xffff)!=0x28)&&
+       (((buf[0]>>16)&0xffff)!=0x100)&&
+       (((buf[0]>>16)&0xffff)!=0x101)&&
+       (((buf[0]>>16)&0xffff)!=0x103)
+       )return 0;
+   switch(buf[0]>>16){
+      case 0x18: nv3_chiptype=Riva128; break;
+      case 0x20:
+      case 0x29: case 0x2C: case 0x2D: case 0xA0:
+      case 0x28: nv3_chiptype=RivaTNT; break;
+      case 0x101: case 0x103:
+      case 0x100: nv3_chiptype=GEFORCE; break;
+      default: nv3_chiptype=Riva128;
+   };
    MMIOBASE=buf[4]&0xffffff00;
    LINEARBASE=buf[5]&0xffffff00;
    nv3_init(0,0,0);
@@ -460,21 +491,11 @@ static void nv3_setwrpage(int page)
 static void nv3_setdisplaystart(int address)
 {  unsigned char byte;
   address=address >> 2;
-  __svgalib_outCR(0x0d,address&0xff);
-  __svgalib_outCR(0x0c,(address>>8)&0xff);
+  __svgalib_outcrtc(0x0d,address&0xff);
+  __svgalib_outcrtc(0x0c,(address>>8)&0xff);
   byte=PCRTC_Read(REPAINT0) & 0xe0;
   PCRTC_Write(REPAINT0,((address>>16)&0x1f)|byte);
   
-/*  inb(0x3da);
-  outb(0x3c0,0x13);
-  outb(0x3c0,(address&3) << 1);
-  address = address << 2 ;
-  outw(0x3d4, (address & 0x00FF00) | 0x0C);
-  outw(0x3d4, ((address & 0x00FF) << 8) | 0x0D);
-
-  byte=PCRTC_Read(REPAINT0) & 0xe0;
-  PCRTC_Write(REPAINT0,(address&0x0f)|byte);
-*/
 }
 
 
@@ -483,7 +504,7 @@ static void nv3_setdisplaystart(int address)
 static void nv3_setlogicalwidth(int width)
 {  int byte ;
 
-__svgalib_outCR(0x13,(width >> 3)&0xff);
+  __svgalib_outcrtc(0x13,(width >> 3)&0xff);
   byte=PCRTC_Read(REPAINT0) & 0x1f;
   PCRTC_Write(REPAINT0,SetBitField(width,13:11,7:5)|byte);
 
@@ -546,21 +567,91 @@ DriverSpecs __svgalib_nv3_driverspecs =
 /* Initialize chipset (called after detection) */
 
 static int nv3_init(int force, int par1, int par2)
-{   nv3_unlock();
+{   
+    char *architectures[3]={"nv3, Riva128/Riva128ZX",
+                            "nv4, RivaTNT/RivaTNT2", 
+                            "nv10, GeForce"};
+    
+    nv3_unlock();
+    
+    if(MMIOBASE==0) {
+       unsigned long buf[64];
+       int _ioperm=0,found;
+       
+       if (getenv("IOPERM") == NULL) {
+         _ioperm=1;
+         if (iopl(3) < 0) {
+           printf("svgalib: nv3: cannot get I/O permissions\n");
+           exit(1);
+         }
+       }
+       found=__svgalib_pci_find_vendor_vga(0x12d2,buf,0);
+       if (_ioperm) iopl(0);
+       if (found)return 0;
+       if (found) {
+          found=__svgalib_pci_find_vendor_vga(0x10de,buf,0);
+          if (_ioperm) iopl(0);
+          if(found)return 0;
+       };
+       switch(buf[0]>>16){
+          case 0x18: nv3_chiptype=Riva128; break;
+          case 0x20:
+          case 0x29: case 0x2C: case 0x2D: case 0xA0:
+          case 0x28: nv3_chiptype=RivaTNT; break;
+          case 0x101: case 0x103:
+          case 0x100: nv3_chiptype=GEFORCE; break;
+          default: nv3_chiptype=Riva128;
+       };
+       MMIOBASE=buf[4]&0xffffff00;
+       LINEARBASE=buf[5]&0xffffff00;
+    };
     if (force) {
 	nv3_memory = par1;
 	nv3_chiptype = par2;
     };
     MapDevice(PRAMDAC,regBase);
     MapDevice(PFB,regBase);
-        
+    MapDevice(PEXTDEV,regBase);
+     
     if(!force){
-       nv3_memory=1024<<(PFB_Read(BOOT_0)&3); 
-       nv3_chiptype=0;
+       int boot0;
+       
+       boot0=PFB_Read(BOOT_0);
+       switch(nv3_chiptype){
+          case Riva128:
+                 if(boot0&0x20)nv3_memory=8192; else nv3_memory=1024<<(boot0&3); 
+                 if(nv3_memory==1024)nv3_memory=8192;
+                 break;
+          case RivaTNT: 
+                 nv3_memory=2048<<(boot0&3); 
+                 if(nv3_memory==2048)nv3_memory=32768;
+                 break;
+          case GEFORCE:
+             	 nv3_memory=8192; /* do this later */
+                 break;
+       };
     };
+
+    {
+       int temp;
+       
+       temp=PEXTDEV_Read(0);
+       switch(nv3_chiptype){
+          case Riva128:
+             PLL_INPUT_FREQ= (temp&0x20) ? 14318 : 13500;
+             break;
+          case RivaTNT:
+          case GEFORCE:
+             PLL_INPUT_FREQ= (temp&0x40) ? 14318 : 13500;
+             break;
+       };
+       
+    };
+
     if (__svgalib_driver_report) {
-	printf("Using RIVA 128 driver, %iKB.\n",nv3_memory);
+	printf("Using RIVA driver, %iKB, Type:%s.\n",nv3_memory,architectures[nv3_chiptype]);
     };
+
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = nv3_memory;
     cardspecs->maxPixelClock4bpp = 75000;	
@@ -577,6 +668,14 @@ static int nv3_init(int force, int par1, int par2)
     cardspecs->mapHorizontalCrtc = nv3_map_horizontal_crtc;
     cardspecs->matchProgrammableClock=nv3_match_programmable_clock;
     __svgalib_driverspecs = &__svgalib_nv3_driverspecs;
+    
+    if((nv3_chiptype==RivaTNT)||(nv3_chiptype=GEFORCE))
+      __svgalib_driverspecs->unlock=nv4_unlock;
+    
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+    __svgalib_linear_mem_base=LINEARBASE;
+    __svgalib_linear_mem_size=nv3_memory*0x400;
     return 0;
 }
 
