@@ -10,7 +10,8 @@
  */
 
 /*
- * Last modified: 1 Sep 1997
+ * Mar 1999 (Eduardo ...)
+ * Recognizes Trio3D as Trio64
  *
  * Sep 1997 (Greg Alexander):
  * Recognizes S3Trio64V2/DX cards as Trio64's.
@@ -89,9 +90,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>		/* iopl() */
-#ifdef USEGLIBC
-#include <sys/io.h>             /* glibc iopl() */
-#endif
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -101,6 +99,7 @@
 #include "vgaregs.h"
 #include "interface.h"
 #include "8514a.h"
+#include "vgapci.h"
 
 /* no acceleration as of now. */
 #undef S3_USE_GRAPHIC_ENGINE
@@ -110,7 +109,7 @@
 #undef S3_KLUDGE_PAGE_MODE
 
 /* use alternate 'linear' banking method for 864+ */
-#define S3_LINEAR_MODE_BANKING_864
+#undef S3_LINEAR_MODE_BANKING_864
 
 #ifdef __alpha__		/* no good for alpha's */
 #undef S3_LINEAR_MODE_BANKING_864
@@ -122,7 +121,7 @@
  * XXXX does not work with console switching and might be incompatible with
  *      S3_LINEAR_MODE_BANKING_864.
  */
-#undef S3_LINEAR_SUPPORT
+#define S3_LINEAR_SUPPORT
 
 /* supports 16 colors */
 #define S3_16_COLORS
@@ -254,16 +253,28 @@ static DacMethods *dac_used;
 static ClockChipMethods *clk_used;
 static int dac_speed = 0;
 
+int s3Mclk = 0;
+
 /* forward declaration. */
 extern DriverSpecs __svgalib_s3_driverspecs;
 
 static int s3_init(int, int, int);
 static void s3_setpage(int page);
+#ifdef S3_LINEAR_MODE_BANKING_864
 static void s3_setpage864(int page);
+#endif
 
 #ifdef S3_LINEAR_SUPPORT
+static int s3_cr40;
+static int s3_cr54;
+static int s3_cr58;
+static int s3_cr59;
+static int s3_cr5A;
+static int s3_linear_opt = 0;
 static int s3_linear_addr = 0;
-static void s3_setlinear(int addr);
+static int s3_linear_base = 0;
+static void s3_linear_enable(void);
+static void s3_linear_disable(void);
 #endif
 
 static void nothing(void)
@@ -391,9 +402,11 @@ static void s3_getmodeinfo(int mode, vga_modeinfo * modeinfo)
     modeinfo->flags &= ~HAVE_RWPAGE;
     modeinfo->flags |= HAVE_EXT_SET;
 #ifdef S3_LINEAR_SUPPORT
-    modeinfo->flags |= CAPABLE_LINEAR;
-    if (s3_linear_addr)
-	modeinfo->flags |= IS_LINEAR;
+    if (modeinfo->bytesperpixel >= 1) {
+    	modeinfo->flags |= CAPABLE_LINEAR;
+    	if (s3_linear_addr)
+	    modeinfo->flags |= IS_LINEAR;
+    }
 #endif
 
     modeinfo->memory = s3_memory;
@@ -529,7 +542,8 @@ static int s3_saveregs(unsigned char regs[])
 
     /* get S3 ext. SR registers */
     /* if (s3_chiptype >= S3_864) { */
-    if (s3_chiptype == S3_765) {/* SL: actually Trio32/64/V+ */
+    if (s3_chiptype == S3_TRIO32 || s3_chiptype == S3_TRIO64
+    	|| s3_chiptype == S3_765) {/* SL: actually Trio32/64/V+ */
 	regs[S3_SR08] = __svgalib_inSR(0x08);
 	__svgalib_outSR(0x08, 0x06);	/* unlock extended seq regs */
 	regs[S3_SR09] = __svgalib_inSR(0x09);
@@ -640,7 +654,7 @@ static void s3_setregs(const unsigned char regs[], int mode)
     }
 #ifdef S3_LINEAR_SUPPORT
     if (mode == TEXT && s3_linear_addr)
-	s3_setlinear(0);	/* make sure linear is off */
+	s3_linear_disable();	/* make sure linear is off */
 #endif
 
     /* restore CR38/39 (may lock other regs) */
@@ -673,7 +687,7 @@ static void s3_initializemode(unsigned char *moderegs,
     moderegs[VGA_CR13] = modeinfo->lineWidth >> 3;
     moderegs[VGA_CR17] = 0xE3;
 
-    if (modeinfo->lineWidth == 2048)
+    if (modeinfo->lineWidth / modeinfo->bytesPerPixel == 2048)
 	moderegs[S3_CR31] = 0x8F;
     else
 	moderegs[S3_CR31] = 0x8D;
@@ -769,7 +783,10 @@ static void s3_initializemode(unsigned char *moderegs,
 	else
 	    s3_8514regs[S3_ADVFUNC_CNTL] |= 0x0001;
 #endif
-    }
+    } else if (modeinfo->lineWidth / modeinfo->bytesPerPixel == 1024)
+    	s3_8514regs[S3_ADVFUNC_CNTL] = 0x0007;
+    else
+    	s3_8514regs[S3_ADVFUNC_CNTL] = 0x0003;
 
     moderegs[S3_CR44] = 0;
     /* Skip CR45, 'hi/truecolor cursor color enable'. */
@@ -792,7 +809,7 @@ static void s3_initializemode(unsigned char *moderegs,
 	    break;
 	}
 
-	switch (modeinfo->lineWidth) {
+	switch (modeinfo->lineWidth / modeinfo->bytesPerPixel) {
 	case 640:
 	    moderegs[S3_CR50] |= 0x40;
 	    break;
@@ -833,16 +850,16 @@ static void s3_initializemode(unsigned char *moderegs,
 		clock = modetiming->pixelClock *
 		    modeinfo->bytesPerPixel;
 	    }
-	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO64 
-		|| s3_chiptype == S3_765)
+	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO32)
 		clock *= 2;
-	    if (s3_chiptype == S3_801 || s3_chiptype == S3_805)
+	    if (s3Mclk > 0)
+	    	mclk = s3Mclk;
+	    else if (s3_chiptype == S3_801 || s3_chiptype == S3_805)
 		mclk = 50000;	/* Assumption. */
 	    else
 		mclk = 60000;	/* Assumption. */
 	    m = (int) ((mclk / 1000.0 * .72 + 16.867) * 89.736 / (clock / 1000.0 + 39) - 21.1543);
-	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO64
-		|| s3_chiptype == S3_765)
+	    if (s3_memory < 2048 || s3_chiptype == S3_TRIO32)
 		m /= 2;
 	    if (m > 31)
 		m = 31;
@@ -863,7 +880,6 @@ static void s3_initializemode(unsigned char *moderegs,
 	moderegs[S3_CR55] &= 0x08;
 	moderegs[S3_CR55] |= 0x40;
 
-	moderegs[S3_CR58] = 0;	/* s3SAM256 */
 #ifdef S3_LINEAR_MODE_BANKING_864
 	if (s3_chiptype >= S3_864) {
 	    if (modeinfo->bitsPerPixel >= 8) {
@@ -882,8 +898,10 @@ static void s3_initializemode(unsigned char *moderegs,
 	    }
 	}
 #endif
-	moderegs[S3_CR59] = 0;
-	moderegs[S3_CR5A] = 0x0A;
+#ifdef S3_LINEAR_SUPPORT
+	moderegs[S3_CR59] = s3_cr59;
+	moderegs[S3_CR5A] = s3_cr5A;
+#endif
 
 	/* Extended CRTC timing. */
 	moderegs[S3_CR5E] =
@@ -898,24 +916,29 @@ static void s3_initializemode(unsigned char *moderegs,
 		((((modetiming->CrtcHDisplay >> 3) - 1) & 0x100) >> 7) |
 		((((modetiming->CrtcHSyncStart >> 3) - 1) & 0x100) >> 6) |
 		((modetiming->CrtcHSyncStart & 0x800) >> 7);
+	    if ((modetiming->CrtcHSyncEnd >> 3) - (modetiming->CrtcHSyncStart >> 3) > 64)
+	    	i |= 0x08;
+	    if ((modetiming->CrtcHSyncEnd >> 3) - (modetiming->CrtcHSyncStart >> 3) > 32)
+	    	i |= 0x20;
 	    j = ((moderegs[VGA_CR0] + ((i & 0x01) << 8) +
 		  moderegs[VGA_CR4] + ((i & 0x10) << 4) + 1) / 2);
+	    if (j - (moderegs[VGA_CR4] + ((i & 0x10) << 4)) < 4) {
+	    	if (moderegs[VGA_CR4] + ((i & 0x10) << 4) + 4 <= moderegs[VGA_CR0] + ((i & 0x01) << 8))
+	    	    j = moderegs[VGA_CR4] + ((i & 0x10) << 4) + 4;
+	    	else
+	    	    j = moderegs[VGA_CR0] + ((i & 0x01) << 8) + 1;
+	    }
+            
 	    moderegs[S3_CR3B] = j & 0xFF;
 	    i |= (j & 0x100) >> 2;
 	    /* Interlace mode frame offset. */
 	    moderegs[S3_CR3C] = (moderegs[VGA_CR0] + ((i & 0x01) << 8)) / 2;
-	    moderegs[S3_CR5D] &= ~0x57;
-	    moderegs[S3_CR5D] |= i;
+	    moderegs[S3_CR5D] = (moderegs[S3_CR5D] & 0x80) | i;
 	}
 
 	{
 	    int i;
-#if 0				/* This seems wrong. Why doesn't XF86_S3 have problems? */
-	    if (s3_chiptype == S3_864 && s3_memory > 1024)
-		i = modetiming->HDisplay *
-		    modeinfo->bytesPerPixel / 8 + 1;
-	    /* else */
-#endif
+
 	    if (modeinfo->bitsPerPixel < 8)
 		i = modetiming->HDisplay / 4 + 1;
 	    else
@@ -1044,6 +1067,11 @@ static void s3_initializemode(unsigned char *moderegs,
 	    moderegs[S3_CR42] |= 0x02;
 	}
     }
+#ifdef S3_LINEAR_SUPPORT
+    s3_cr58 = moderegs[S3_CR58];
+    s3_cr40 = moderegs[S3_CR40];
+    s3_cr54 = moderegs[S3_CR54];
+#endif
     if (clk_used == &__svgalib_I2061A_clockchip_methods &&
 	(modetiming->flags & USEPROGRCLOCK)) {
 	/* Clock select. */
@@ -1191,13 +1219,13 @@ static void s3_setpage(int page)
 	page *= 4;
 #endif
     s3_unlock();
-    outb(0x3d4, 0x35);
-    outb(0x3d5, (inb(0x3d5) & 0xF0) | (page & 0x0F));
+    outb(CRT_IC, 0x35);
+    outb(CRT_DC, (inb(CRT_DC) & 0xF0) | (page & 0x0F));
     if (s3_chiptype >= S3_801) {
-	outb(0x3d4, 0x51);
-	outb(0x3d5, (inb(0x3d5) & ~0x0C) | ((page & 0x30) >> 2));
+	outb(CRT_IC, 0x51);
+	outb(CRT_DC, (inb(CRT_DC) & ~0x0C) | ((page & 0x30) >> 2));
     }
-    inb(0x3d5);			/* ARI: Ferraro says: required for first generation 911 only */
+    inb(CRT_DC);			/* ARI: Ferraro says: required for first generation 911 only */
     s3_lock();
 }
 
@@ -1213,8 +1241,8 @@ static void s3_setpage864(int page)
 {
     s3_unlock();
     /* "Linear" mode banking. */
-    outb(0x3d4, 0x6A);
-    outb(0x3d5, (inb(0x3d5) & ~0x3F) | page);
+    outb(CRT_IC, 0x6A);
+    outb(CRT_DC, (inb(CRT_DC) & ~0x3F) | page);
     s3_lock();
 }
 
@@ -1235,18 +1263,18 @@ static void s3_setdisplaystart(int address)
 	address *= 4;
 #endif
     s3_unlock();
-    outw(0x3d4, 0x0d | ((address << 6) & 0xff00));	/* sa2-sa9 */
-    outw(0x3d4, 0x0c | ((address >> 2) & 0xff00));	/* sa10-sa17 */
+    outw(CRT_IC, 0x0d | ((address << 6) & 0xff00));	/* sa2-sa9 */
+    outw(CRT_IC, 0x0c | ((address >> 2) & 0xff00));	/* sa10-sa17 */
     inb(0x3da);			/* set ATC to addressing mode */
-    outb(0x3c0, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
+    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
 
-    outb(0x3d4, 0x31);
-    outb(0x3d5, (inb(0x3d5) & ~0x30) | ((address & 0xc0000) >> 14));
+    outb(CRT_IC, 0x31);
+    outb(CRT_DC, (inb(CRT_DC) & ~0x30) | ((address & 0xc0000) >> 14));
     if (s3_chiptype >= S3_801) {
-	outb(0x3d4, 0x51);
-	outb(0x3d5, (inb(0x3d5) & ~0x03) | ((address & 0x300000) >> 20));
+	outb(CRT_IC, 0x51);
+	outb(CRT_DC, (inb(CRT_DC) & ~0x03) | ((address & 0x300000) >> 20));
     }
     s3_lock();
 }
@@ -1262,26 +1290,41 @@ static void s3_setlogicalwidth(int width)
 }
 
 #ifdef S3_LINEAR_SUPPORT
-static void s3_setlinear(int addr)
+static void s3_linear_enable(void)
 {
-    /* printf("s3_setlinear(0x%x) called\n", addr); */
     s3_unlock();
-
-    outb(0x3d4, 0x58);
-    if (addr) {
-	int m;
-
-	m = s3_memory >> 10;	/* XXXX up to 4M */
-	if (!m)
-	    m = 1;
-	outb(0x3d5, inb(0x3d5) | 0x10 | m);	/* enable linear mode */
-	__svgalib_outCR(0x59, (addr >> 8) & 0x03);
-	__svgalib_outCR(0x5A, (addr) & 0xFF);
-    } else {
-	outb(0x3d5, inb(0x3d5) & ~0x13);	/* disable linear mode */
-	__svgalib_outCR(0x59, 0x00);
-	__svgalib_outCR(0x5A, 0x0A);
+    
+    if (s3_chiptype > S3_924) {
+    	int i;
+    	outb (CRT_IC, 0x40);
+    	i = (s3_cr40 & 0xf6) | 0x0a;
+    	outb (CRT_DC, (unsigned char) i);
+    	outb (CRT_IC, 0x58);
+    	outb (CRT_DC, s3_linear_opt | s3_cr58);
+    	if (s3_chiptype > S3_928) {
+    	    outb (CRT_IC, 0x54);
+    	    outb (CRT_DC, (s3_cr54 + 0x07));
+    	}
     }
+    
+    s3_lock();
+}
+
+static void s3_linear_disable(void)
+{
+    s3_unlock();
+    
+    if (s3_chiptype > S3_924) {
+    	if (s3_chiptype > S3_928) {
+    	    outb (CRT_IC, 0x54);
+    	    outb (CRT_DC, s3_cr54);
+    	}
+    	outb (CRT_IC, 0x58);
+    	outb (CRT_DC, s3_cr58);
+    	outb (CRT_IC, 0x40);
+    	outb (CRT_DC, s3_cr40);
+    }
+    
     s3_lock();
 }
 
@@ -1289,11 +1332,8 @@ static void s3_setlinear(int addr)
 
 static int s3_linear(int op, int param)
 {
-    /* printf("s3_linear(%d,0x%x) called\n", op, param); */
-#if 0    
-    if (s3_chiptype != S3_805)
-	return -1;
-#endif
+    if (op == LINEAR_QUERY_BASE)
+    	return s3_linear_base;
     if (op == LINEAR_QUERY_GRANULARITY) {
 	switch (s3_memory) {
 	case 4096:
@@ -1307,12 +1347,12 @@ static int s3_linear(int op, int param)
 	return 256;
     else if (op == LINEAR_ENABLE) {
 	s3_setpage(0);
-	s3_setlinear(param >> 16);
+	s3_linear_enable();
 	s3_linear_addr = param;
 	return 0;
     } else if (op == LINEAR_DISABLE) {
 	s3_setpage(0);
-	s3_setlinear(0);
+	s3_linear_disable();
 	s3_linear_addr = 0;
 	return 0;
     } else
@@ -1571,6 +1611,7 @@ static int s3_init(int force, int par1, int par2)
 	    case 0x3DE0: /* ViRGE/VX ID */
 	    case 0x31E0: /* ViRGE ID */
 	    case 0x01E0: /* S3Trio64V2/DX ... any others? */
+            case 0x04E0:
 	    case 0x11E0:
 		if (rev & 0x0400)
 		    s3_chiptype = S3_765;
@@ -1634,8 +1675,8 @@ static int s3_init(int force, int par1, int par2)
 	       s3_memory);
 	if (s3_flags & S3_OLD_STEPPING)
 	    printf("svgalib: Chip revision cannot handle modes with width 1152.\n");
-	if (s3_chiptype > S3_864) {
-	    printf("svgalib: s3: chipsets newer than S3-864 is not supported well yet.\n");
+	if (s3_chiptype > S3_TRIO64) {
+	    printf("svgalib: s3: chipsets newer than S3 Trio64 is not supported well yet.\n");
 	}
     }
 /* begin: Initialize cardspecs. */
@@ -1645,6 +1686,47 @@ static int s3_init(int force, int par1, int par2)
 	if (0 > iopl(3))
 	    printf("svgalib: s3: cannot get I/O permissions for 8514.");
     }
+#ifdef S3_LINEAR_SUPPORT
+    if (s3_chiptype > S3_805) {
+    	int found_pciconfig;
+    	unsigned long pci_conf[64];
+    	
+    	found_pciconfig = __svgalib_pci_find_vendor_vga(0x5333, pci_conf, 0);
+    	if (!found_pciconfig)
+    	    s3_linear_base = pci_conf[4] & 0xFF800000;
+    }
+    
+    s3_cr59 = s3_linear_base >> 24;
+    s3_cr5A = (s3_linear_base >> 16);
+    if (! (s3_cr59 | s3_cr5A)) {
+    	s3_cr59 = __svgalib_inCR(0x59);
+    	s3_cr5A = __svgalib_inCR(0x5A);
+    	if (!s3_cr59) {
+    	    s3_cr59 =  0xF3000000 >> 24;
+    	    s3_cr5A = (0xF3000000 >> 16);
+    	}
+    	s3_linear_base = (s3_cr59 << 24) | (s3_cr5A << 16);
+    }
+    s3_linear_opt |= 0x10;
+    switch (s3_memory) {
+    	case 512 :
+    	case 1024 : 
+    	    s3_linear_opt |= 0x01;
+    	    break;
+    	case 2048 :
+    	    s3_linear_opt |= 0x02;
+    	    break;
+    	case 3072 :
+    	case 4096 :
+    	case 6144 :
+    	case 8192 :
+    	    s3_linear_opt |= 0x03;
+    	    break;
+    	default :
+    	    s3_linear_opt = 0x14;	/* like XFree */
+    }
+#endif
+    
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = s3_memory;
     cardspecs->nClocks = 0;
@@ -1719,5 +1801,12 @@ static int s3_init(int force, int par1, int par2)
 
     __svgalib_driverspecs = &__svgalib_s3_driverspecs;
 
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+#ifdef S3_LINEAR_SUPPORT
+    __svgalib_linear_mem_base=s3_linear_base;
+    __svgalib_linear_mem_size=s3_memory*0x400;
+#endif
+
     return 0;
 }
